@@ -43,6 +43,8 @@ fn main() {
   let mut focus_all_window =
     tray_menu.add_item(MenuItemAttributes::new("Focus window").with_enabled(false));
 
+  let change_menu = tray_menu.add_item(MenuItemAttributes::new("Change menu"));
+
   // inject submenu into tray_menu
   tray_menu.add_submenu("Sub menu", true, submenu);
 
@@ -108,7 +110,9 @@ fn main() {
     };
 
     match event {
-      Event::WindowEvent { event, window_id } => {
+      Event::WindowEvent {
+        event, window_id, ..
+      } => {
         if event == WindowEvent::CloseRequested {
           let mut open_new_window_element = open_new_window_element.clone();
           // Remove window from our hashmap
@@ -156,6 +160,13 @@ fn main() {
           // tell our app to close at the end of the loop.
           *control_flow = ControlFlow::Exit;
         }
+
+        if menu_id == change_menu.clone().id() {
+          let mut tray_menu = Menu::new();
+          tray_menu.add_item(MenuItemAttributes::new("Quit"));
+          system_tray.set_menu(&tray_menu);
+        }
+
         println!("Clicked on {:?}", menu_id);
       }
       _ => (),
