@@ -13,16 +13,15 @@
 //!
 //! And the following platform-specific module:
 //!
+//! - `global_shortcut` (available on `windows`, `unix`, `macos`)
 //! - `run_return` (available on `windows`, `unix`, `macos`, and `android`)
-//! - `status_bar` (available on `windows`, `unix`, and `macos`)
 //!
 //! However only the module corresponding to the platform you're compiling to will be available.
 
 pub mod android;
+pub mod global_shortcut;
 pub mod ios;
 pub mod macos;
+pub mod run_return;
 pub mod unix;
 pub mod windows;
-
-pub mod run_return;
-pub mod system_tray;
