@@ -5,12 +5,7 @@
 
 use std::{os::raw::c_void, path::Path};
 
-use libc;
-use winapi::shared::{
-  minwindef::WORD,
-  windef::{HMENU, HWND},
-};
-
+pub use crate::platform_impl::hit_test;
 use crate::{
   dpi::PhysicalSize,
   event::DeviceId,
@@ -19,6 +14,14 @@ use crate::{
   platform_impl::{EventLoop as WindowsEventLoop, Parent, WinIcon},
   window::{BadIcon, Icon, Theme, Window, WindowBuilder},
 };
+use libc;
+use winapi::{
+  shared::{
+    minwindef::{self, WORD},
+    windef::{HMENU, HWND},
+  },
+  um::winuser,
+};
 
 /// Additional methods on `EventLoop` that are specific to Windows.
 pub trait EventLoopExtWindows {
@@ -109,6 +112,12 @@ pub trait WindowExtWindows {
   /// this function can be called to reset the dead key state so that
   /// follow-up text input won't be affected by the dead key.
   fn reset_dead_keys(&self);
+
+  /// Starts the resizing drag from given edge
+  fn begin_resize_drag(&self, edge: isize);
+
+  /// Whether to show the window icon in the taskbar or not.
+  fn set_skip_taskbar(&self, skip: bool);
 }
 
 impl WindowExtWindows for Window {
@@ -143,6 +152,30 @@ impl WindowExtWindows for Window {
   fn reset_dead_keys(&self) {
     self.window.reset_dead_keys();
   }
+
+  #[inline]
+  fn begin_resize_drag(&self, edge: isize) {
+    unsafe {
+      let point = {
+        let mut pos = std::mem::zeroed();
+        winuser::GetCursorPos(&mut pos);
+        pos
+      };
+
+      winuser::ReleaseCapture();
+      winuser::PostMessageW(
+        self.hwnd() as _,
+        winuser::WM_NCLBUTTONDOWN,
+        edge as minwindef::WPARAM,
+        &point as *const _ as minwindef::LPARAM,
+      );
+    }
+  }
+
+  #[inline]
+  fn set_skip_taskbar(&self, skip: bool) {
+    self.window.set_skip_taskbar(skip);
+  }
 }
 
 /// Additional methods on `WindowBuilder` that are specific to Windows.
@@ -192,6 +225,9 @@ pub trait WindowBuilderExtWindows {
 
   /// Forces a theme or uses the system settings if `None` was provided.
   fn with_theme(self, theme: Option<Theme>) -> WindowBuilder;
+
+  /// Whether to create the window icon with the taskbar icon or not.
+  fn with_skip_taskbar(self, skip: bool) -> WindowBuilder;
 }
 
 impl WindowBuilderExtWindows for WindowBuilder {
@@ -236,6 +272,12 @@ impl WindowBuilderExtWindows for WindowBuilder {
     self.platform_specific.preferred_theme = theme;
     self
   }
+
+  #[inline]
+  fn with_skip_taskbar(mut self, skip: bool) -> WindowBuilder {
+    self.platform_specific.skip_taskbar = skip;
+    self
+  }
 }
 
 /// Additional methods on `MonitorHandle` that are specific to Windows.
