@@ -40,6 +40,8 @@ use super::{
 };
 
 pub struct EventLoopWindowTarget<T> {
+  /// Gdk display
+  pub(crate) display: gdk::Display,
   /// Gtk application
   pub(crate) app: gtk::Application,
   /// Window Ids of the application
@@ -52,12 +54,24 @@ pub struct EventLoopWindowTarget<T> {
 impl<T> EventLoopWindowTarget<T> {
   #[inline]
   pub fn available_monitors(&self) -> VecDeque<MonitorHandle> {
-    todo!()
+    let mut handles = VecDeque::new();
+    let display = &self.display;
+    let numbers = display.get_n_monitors();
+
+    for i in 0..numbers {
+      let monitor = MonitorHandle::new(&display, i);
+      handles.push_back(monitor);
+    }
+
+    handles
   }
 
   #[inline]
   pub fn primary_monitor(&self) -> Option<RootMonitorHandle> {
-    todo!()
+    let screen = self.display.get_default_screen();
+    let number = screen.get_primary_monitor();
+    let handle = MonitorHandle::new(&self.display, number);
+    Some(RootMonitorHandle { inner: handle })
   }
 }
 
@@ -85,7 +99,10 @@ impl<T: 'static> EventLoop<T> {
 
     // Create event loop window target.
     let (window_requests_tx, window_requests_rx) = glib::MainContext::channel(Priority::default());
+    let display = gdk::Display::get_default()
+      .expect("GdkDisplay not found. This usually means `gkt_init` hasn't called yet.");
     let window_target = EventLoopWindowTarget {
+      display,
       app,
       windows: Rc::new(RefCell::new(HashSet::new())),
       window_requests_tx,
@@ -149,7 +166,6 @@ impl<T: 'static> EventLoop<T> {
 
     // Window Request
     let app = window_target.p.app.clone();
-    let windows = window_target.p.windows.clone();
     let window_requests_tx = window_target.p.window_requests_tx.clone();
     self
       .window_requests_rx
@@ -243,9 +259,7 @@ impl<T: 'static> EventLoop<T> {
               }
             }
             WindowRequest::UserAttention(request_type) => {
-              if request_type.is_some() {
-                window.set_urgency_hint(true)
-              }
+              window.set_urgency_hint(request_type.is_some())
             }
             WindowRequest::SetSkipTaskbar(skip) => window.set_skip_taskbar_hint(skip),
             WindowRequest::CursorIcon(cursor) => {
@@ -303,18 +317,16 @@ impl<T: 'static> EventLoop<T> {
               };
             }
             WindowRequest::WireUpEvents => {
-              let windows_rc = windows.clone();
               let tx_clone = event_tx.clone();
 
               window.connect_delete_event(move |_, _| {
-                windows_rc.borrow_mut().remove(&id);
                 if let Err(e) = tx_clone.send(Event::WindowEvent {
                   window_id: RootWindowId(id),
                   event: WindowEvent::CloseRequested,
                 }) {
                   log::warn!("Failed to send window close event to event channel: {}", e);
                 }
-                Inhibit(false)
+                Inhibit(true)
               });
 
               let tx_clone = event_tx.clone();
