@@ -22,10 +22,10 @@ use crate::{
   window::{CursorIcon, Fullscreen, UserAttentionType, WindowAttributes},
 };
 
-use super::{event_loop::EventLoopWindowTarget, menu, monitor::MonitorHandle};
-
-#[derive(Clone, Default)]
-pub struct PlatformSpecificWindowBuilderAttributes {}
+use super::{
+  event_loop::EventLoopWindowTarget, menu, monitor::MonitorHandle,
+  PlatformSpecificWindowBuilderAttributes,
+};
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct WindowId(pub(crate) u32);
@@ -99,7 +99,7 @@ impl Window {
   pub(crate) fn new<T>(
     event_loop_window_target: &EventLoopWindowTarget<T>,
     attributes: WindowAttributes,
-    _pl_attribs: PlatformSpecificWindowBuilderAttributes,
+    pl_attribs: PlatformSpecificWindowBuilderAttributes,
   ) -> Result<Self, RootOsError> {
     let app = &event_loop_window_target.app;
     let window_requests_tx = event_loop_window_target.window_requests_tx.clone();
@@ -199,6 +199,7 @@ impl Window {
 
     // Rest attributes
     window.set_title(&attributes.title);
+    // TODO set it with Fullscreen enum
     if attributes.fullscreen.is_some() {
       window.fullscreen();
     }
@@ -207,10 +208,6 @@ impl Window {
     }
     window.set_visible(attributes.visible);
     window.set_decorated(attributes.decorations);
-    if attributes.focus {
-      // FIXME: replace with present_with_timestamp
-      window.present();
-    }
 
     if !attributes.decorations && attributes.resizable {
       window.add_events(EventMask::POINTER_MOTION_MASK | EventMask::BUTTON_MOTION_MASK);
@@ -250,8 +247,6 @@ impl Window {
       window.hide();
     }
 
-    window.set_skip_taskbar_hint(attributes.skip_taskbar);
-
     let w_pos = window.get_position();
     let position: Rc<(AtomicI32, AtomicI32)> = Rc::new((w_pos.0.into(), w_pos.1.into()));
     let position_clone = position.clone();
@@ -291,10 +286,7 @@ impl Window {
       scale_factor_clone.store(window.get_scale_factor(), Ordering::Release);
     });
 
-    if let Err(e) = window_requests_tx
-      .clone()
-      .send((window_id, WindowRequest::WireUpEvents))
-    {
+    if let Err(e) = window_requests_tx.send((window_id, WindowRequest::WireUpEvents)) {
       log::warn!("Fail to send wire up events request: {}", e);
     }
 
@@ -323,7 +315,8 @@ impl Window {
     });
 
     window.queue_draw();
-    Ok(Self {
+
+    let win = Self {
       window_id,
       window,
       window_requests_tx,
@@ -335,7 +328,11 @@ impl Window {
       maximized,
       minimized,
       fullscreen: RefCell::new(attributes.fullscreen),
-    })
+    };
+
+    win.set_skip_taskbar(pl_attribs.skip_taskbar);
+
+    Ok(win)
   }
 
   pub fn id(&self) -> WindowId {
@@ -593,6 +590,10 @@ impl Window {
     self.menu_bar.show_all();
   }
 
+  pub fn is_menu_visible(&self) -> bool {
+    self.menu_bar.get_visible()
+  }
+
   pub fn set_cursor_icon(&self, cursor: CursorIcon) {
     if let Err(e) = self
       .window_requests_tx
@@ -625,23 +626,39 @@ impl Window {
   }
 
   pub fn current_monitor(&self) -> Option<RootMonitorHandle> {
-    todo!()
+    let screen = self.window.get_display().get_default_screen();
+    let window = self.window.get_window().unwrap();
+    let number = screen.get_monitor_at_window(&window);
+    let handle = MonitorHandle::new(&self.window.get_display(), number);
+    Some(RootMonitorHandle { inner: handle })
   }
 
   #[inline]
   pub fn available_monitors(&self) -> VecDeque<MonitorHandle> {
-    todo!()
+    let mut handles = VecDeque::new();
+    let display = self.window.get_display();
+    let numbers = display.get_n_monitors();
+
+    for i in 0..numbers {
+      let monitor = MonitorHandle::new(&display, i);
+      handles.push_back(monitor);
+    }
+
+    handles
   }
 
   pub fn primary_monitor(&self) -> Option<RootMonitorHandle> {
-    todo!()
+    let screen = self.window.get_display().get_default_screen();
+    let number = screen.get_primary_monitor();
+    let handle = MonitorHandle::new(&self.window.get_display(), number);
+    Some(RootMonitorHandle { inner: handle })
   }
 
   pub fn raw_window_handle(&self) -> raw_window_handle::RawWindowHandle {
     todo!()
   }
 
-  pub fn set_skip_taskbar(&self, skip: bool) {
+  pub(crate) fn set_skip_taskbar(&self, skip: bool) {
     if let Err(e) = self
       .window_requests_tx
       .send((self.window_id, WindowRequest::SetSkipTaskbar(skip)))
@@ -656,6 +673,7 @@ impl Window {
 unsafe impl Send for Window {}
 unsafe impl Sync for Window {}
 
+#[non_exhaustive]
 pub enum WindowRequest {
   Title(String),
   Position((i32, i32)),
@@ -689,7 +707,7 @@ pub fn hit_test(window: &gdk::Window, cx: f64, cy: f64) -> WindowEdge {
   let (right, bottom) = (left + w, top + h);
   let (cx, cy) = (cx as i32, cy as i32);
 
-  let fake_border = 5; // change this to manipulate how far inside the window, the resize can happen
+  let fake_border = 3; // change this to manipulate how far inside the window, the resize can happen
 
   let display = window.get_display();
 
@@ -742,3 +760,11 @@ pub fn hit_test(window: &gdk::Window, cx: f64, cy: f64) -> WindowEdge {
 
   edge
 }
+
+impl Drop for Window {
+  fn drop(&mut self) {
+    unsafe {
+      self.window.destroy();
+    }
+  }
+}
