@@ -389,7 +389,6 @@ impl UnownedWindow {
     let fullscreen = win_attribs.fullscreen.take();
     let maximized = win_attribs.maximized;
     let visible = win_attribs.visible;
-    let focus = win_attribs.focus;
     let decorations = win_attribs.decorations;
     let inner_rect = win_attribs
       .inner_size
@@ -418,17 +417,6 @@ impl UnownedWindow {
       unsafe { window.ns_window.makeKeyAndOrderFront_(nil) };
     }
 
-    if focus {
-      unsafe {
-        // if visible, we already send it
-        if !visible {
-          window.ns_window.makeKeyAndOrderFront_(nil);
-        }
-        let app: id = msg_send![class!(NSApplication), sharedApplication];
-        let () = msg_send![app, activateIgnoringOtherApps: YES];
-      };
-    }
-
     if maximized {
       window.set_maximized(maximized);
     }
@@ -474,9 +462,7 @@ impl UnownedWindow {
   pub fn set_focus(&self) {
     unsafe {
       let is_minimized: BOOL = msg_send![*self.ns_window, isMiniaturized];
-      let is_visible: BOOL = msg_send![*self.ns_window, isVisible];
-
-      if is_minimized == NO && is_visible == YES {
+      if is_minimized == NO {
         util::set_focus(*self.ns_window);
       }
     }
@@ -1039,7 +1025,10 @@ impl UnownedWindow {
   pub fn show_menu(&self) {}
 
   #[inline]
-  pub fn set_skip_taskbar(&self, _skip: bool) {}
+  pub fn is_menu_visible(&self) -> bool {
+    warn!("`Window::is_menu_visible` always return true on macOS");
+    true
+  }
 
   #[inline]
   // Allow directly accessing the current monitor internally without unwrapping.
