@@ -18,7 +18,8 @@ use std::{
 use winapi::{
   ctypes::c_int,
   shared::{
-    minwindef::{HINSTANCE, LPARAM, UINT, WPARAM},
+    basetsd::LONG_PTR,
+    minwindef::{HINSTANCE, LPARAM, LRESULT, UINT, WPARAM},
     windef::{self, HWND, POINT, POINTS, RECT},
   },
   um::{
@@ -210,12 +211,9 @@ impl Window {
   pub fn set_outer_position(&self, position: Position) {
     let (x, y): (i32, i32) = position.to_physical::<i32>(self.scale_factor()).into();
 
-    let window_state = Arc::clone(&self.window_state);
     let window = self.window.clone();
     self.thread_executor.execute_in_thread(move || {
-      WindowState::set_window_flags(window_state.lock(), window.0, |f| {
-        f.set(WindowFlags::MAXIMIZED, false)
-      });
+      util::set_maximized(window.0, false);
     });
 
     unsafe {
@@ -264,12 +262,9 @@ impl Window {
     let scale_factor = self.scale_factor();
     let (width, height) = size.to_physical::<u32>(scale_factor).into();
 
-    let window_state = Arc::clone(&self.window_state);
     let window = self.window.clone();
     self.thread_executor.execute_in_thread(move || {
-      WindowState::set_window_flags(window_state.lock(), window.0, |f| {
-        f.set(WindowFlags::MAXIMIZED, false)
-      });
+      util::set_maximized(window.0, false);
     });
 
     util::set_inner_size_physical(self.window.0, width, height);
@@ -436,20 +431,12 @@ impl Window {
 
   #[inline]
   pub fn set_maximized(&self, maximized: bool) {
-    let window = self.window.clone();
-    let window_state = Arc::clone(&self.window_state);
-
-    self.thread_executor.execute_in_thread(move || {
-      WindowState::set_window_flags(window_state.lock(), window.0, |f| {
-        f.set(WindowFlags::MAXIMIZED, maximized)
-      });
-    });
+    util::set_maximized(self.window.0, maximized);
   }
 
   #[inline]
   pub fn is_maximized(&self) -> bool {
-    let window_state = self.window_state.lock();
-    window_state.window_flags.contains(WindowFlags::MAXIMIZED)
+    util::is_maximized(self.window.0)
   }
 
   #[inline]
@@ -733,26 +720,6 @@ impl Window {
     self.window_state.lock().current_theme
   }
 
-  #[inline]
-  pub fn set_skip_taskbar(&self, skip: bool) {
-    unsafe {
-      let mut taskbar_list: *mut ITaskbarList = std::mem::zeroed();
-      CoCreateInstance(
-        &CLSID_TaskbarList,
-        ptr::null_mut(),
-        CLSCTX_SERVER,
-        &ITaskbarList::uuidof(),
-        &mut taskbar_list as *mut _ as *mut _,
-      );
-      if skip {
-        (*taskbar_list).DeleteTab(self.hwnd());
-      } else {
-        (*taskbar_list).AddTab(self.hwnd());
-      }
-      (*taskbar_list).Release();
-    }
-  }
-
   #[inline]
   pub fn hide_menu(&self) {
     unsafe {
@@ -769,6 +736,11 @@ impl Window {
     }
   }
 
+  #[inline]
+  pub fn is_menu_visible(&self) -> bool {
+    unsafe { winuser::GetMenu(self.hwnd()) != ptr::null_mut() }
+  }
+
   #[inline]
   pub fn reset_dead_keys(&self) {
     // `ToUnicode` consumes the dead-key by default, so we are constructing a fake (but valid)
@@ -788,6 +760,26 @@ impl Window {
       );
     }
   }
+
+  #[inline]
+  pub(crate) fn set_skip_taskbar(&self, skip: bool) {
+    unsafe {
+      let mut taskbar_list: *mut ITaskbarList = std::mem::zeroed();
+      CoCreateInstance(
+        &CLSID_TaskbarList,
+        std::ptr::null_mut(),
+        CLSCTX_SERVER,
+        &ITaskbarList::uuidof(),
+        &mut taskbar_list as *mut _ as *mut _,
+      );
+      if skip {
+        (*taskbar_list).DeleteTab(self.hwnd() as _);
+      } else {
+        (*taskbar_list).AddTab(self.hwnd() as _);
+      }
+      (*taskbar_list).Release();
+    }
+  }
 }
 
 impl Drop for Window {
@@ -870,7 +862,7 @@ unsafe fn init<T: 'static>(
       parent.unwrap_or(ptr::null_mut()),
       pl_attribs.menu.unwrap_or(ptr::null_mut()),
       libloaderapi::GetModuleHandleW(ptr::null()),
-      ptr::null_mut(),
+      Box::into_raw(Box::new(!attributes.decorations)) as _,
     );
 
     if handle.is_null() {
@@ -969,11 +961,7 @@ unsafe fn init<T: 'static>(
     win.menu = menu::initialize(window_menu, window_handle, menu_handler).map(|m| HMenuWrapper(m));
   }
 
-  if attributes.focus {
-    force_window_active(win.window.0);
-  }
-
-  win.set_skip_taskbar(attributes.skip_taskbar);
+  win.set_skip_taskbar(pl_attribs.skip_taskbar);
 
   Ok(win)
 }
@@ -999,7 +987,7 @@ unsafe fn register_window_class(
   let class = winuser::WNDCLASSEXW {
     cbSize: mem::size_of::<winuser::WNDCLASSEXW>() as UINT,
     style: winuser::CS_HREDRAW | winuser::CS_VREDRAW | winuser::CS_OWNDC,
-    lpfnWndProc: Some(winuser::DefWindowProcW),
+    lpfnWndProc: Some(window_proc),
     cbClsExtra: 0,
     cbWndExtra: 0,
     hInstance: libloaderapi::GetModuleHandleW(ptr::null()),
@@ -1020,6 +1008,44 @@ unsafe fn register_window_class(
   class_name
 }
 
+unsafe extern "system" fn window_proc(
+  window: HWND,
+  msg: UINT,
+  wparam: WPARAM,
+  lparam: LPARAM,
+) -> LRESULT {
+  let mut userdata = winuser::GetWindowLongPtrW(window, winuser::GWL_USERDATA);
+
+  match msg {
+    winuser::WM_NCCALCSIZE => {
+      // Check if userdata is set and if the value of it is true
+      if userdata != 0 && *(userdata as *mut bool) == true {
+        // adjust the maximized borderless window to fill the work area rectangle of the display monitor
+        if util::is_maximized(window) {
+          let monitor = monitor::current_monitor(window);
+          if let Ok(monitor_info) = monitor::get_monitor_info(monitor.hmonitor()) {
+            let params = &mut *(lparam as *mut winuser::NCCALCSIZE_PARAMS);
+            params.rgrc[0] = monitor_info.rcWork;
+          }
+        }
+        0 // must return a value here, otherwise the window won't be borderless (without decorations)
+      } else {
+        winuser::DefWindowProcW(window, msg, wparam, lparam)
+      }
+    }
+    winuser::WM_NCCREATE => {
+      // Set userdata to the value of lparam. This will be cleared on event loop subclassing.
+      if userdata == 0 {
+        let createstruct = &*(lparam as *const winuser::CREATESTRUCTW);
+        userdata = createstruct.lpCreateParams as LONG_PTR;
+        winuser::SetWindowLongPtrW(window, winuser::GWL_USERDATA, userdata);
+      }
+      winuser::DefWindowProcW(window, msg, wparam, lparam)
+    }
+    _ => winuser::DefWindowProcW(window, msg, wparam, lparam),
+  }
+}
+
 struct ComInitialized(*mut ());
 impl Drop for ComInitialized {
   fn drop(&mut self) {
@@ -1106,3 +1132,55 @@ unsafe fn force_window_active(handle: HWND) {
 
   winuser::SetForegroundWindow(handle);
 }
+
+pub fn hit_test(hwnd: HWND, cx: i32, cy: i32) -> LRESULT {
+  use winapi::shared::minwindef::TRUE;
+  use winuser::{
+    GetWindowRect, HTBOTTOM, HTBOTTOMLEFT, HTBOTTOMRIGHT, HTCLIENT, HTLEFT, HTNOWHERE, HTRIGHT,
+    HTTOP, HTTOPLEFT, HTTOPRIGHT,
+  };
+
+  unsafe {
+    let mut window_rect: RECT = mem::zeroed();
+    if GetWindowRect(hwnd, <*mut _>::cast(&mut window_rect)) == TRUE {
+      const CLIENT: i32 = 0b0000;
+      const LEFT: i32 = 00001;
+      const RIGHT: i32 = 0b0010;
+      const TOP: i32 = 0b0100;
+      const BOTTOM: i32 = 0b1000;
+      const TOPLEFT: i32 = TOP | LEFT;
+      const TOPRIGHT: i32 = TOP | RIGHT;
+      const BOTTOMLEFT: i32 = BOTTOM | LEFT;
+      const BOTTOMRIGHT: i32 = BOTTOM | RIGHT;
+
+      let fake_border = 3; // change this to manipulate how far inside the window, the resize can happen
+
+      let RECT {
+        left,
+        right,
+        bottom,
+        top,
+      } = window_rect;
+
+      let hit_result = LEFT * (if cx < (left + fake_border) { 1 } else { 0 })
+        | RIGHT * (if cx >= (right - fake_border) { 1 } else { 0 })
+        | TOP * (if cy < (top + fake_border) { 1 } else { 0 })
+        | BOTTOM * (if cy >= (bottom - fake_border) { 1 } else { 0 });
+
+      match hit_result {
+        CLIENT => HTCLIENT,
+        LEFT => HTLEFT,
+        RIGHT => HTRIGHT,
+        TOP => HTTOP,
+        BOTTOM => HTBOTTOM,
+        TOPLEFT => HTTOPLEFT,
+        TOPRIGHT => HTTOPRIGHT,
+        BOTTOMLEFT => HTBOTTOMLEFT,
+        BOTTOMRIGHT => HTBOTTOMRIGHT,
+        _ => HTNOWHERE,
+      }
+    } else {
+      HTNOWHERE
+    }
+  }
+}
